------------------------------------------------------
-- class Widget
------------------------------------------------------
Widget = {
			name = "",
			triggerScript = { OnLoad="", OnEvent="", OnClick="" }
		};

function Widget:new(o)
	o = o or {};
	setmetatable(o, self);
	self.__index = self;
	o.triggerScript = {};
	return o;
end

function CreateWidget(type, name, style)
	if (name == nil) then
		return nil;
	end

	AddNewWidget(g_AddonName, type, name, style);
	newWidget = nil;
	if (type == "Dialog") then
		newWidget = Widget:new();
	end
	--if (type == "Widget") then
	--	newWidget = Widget:new();
	--end
	if (type == "MessageWidget") then
		newWidget = CreateMessageWidget();
	end
	if (type == "Container" or type == "container") then
		newWidget = CreateContainer();
	end
	if (type == "Button" or type == "button") then
		if (style == nil) then
			newWidget = CreateButton();
		else
			if (style == "radiobutton") then
				newWidget = CreateRadioButton();
			elseif (style == "checkbox") then
				newWidget = CreateCheckButton();
			else
				newWidget = CreateButton();
			end
		end
	end
	if (type == "TabButton" or type == "tabbutton") then
		newWidget = CreateTabButton();
	end
	if (type == "Tab" or type == "tab") then
		newWidget = CreateTab();
	end
	if (type == "TabSheet" or type == "tabsheet") then
		newWidget = CreateTabSheet();
	end
	if (type == "TabPage" or type == "tabpage") then
		newWidget = CreateTabPage();
	end
	if (type == "CheckButton") then
		newWidget = CreateCheckButton();
	end
	if (type == "RadioButton") then
		newWidget = CreateRadioButton();
	end
	if (type == "SimpleHTML" or type == "htmlview") then
		newWidget = CreateSimpleHTML();
	end
	if (type == "ComboBox" or type == "combobox") then
		newWidget = CreateComboBox();
	end
	if (type == "EditBox" or type == "editbox") then
		newWidget = CreateEditBox();
	end
	if (type == "Gauge" or type == "gauge") then
		newWidget = CreateGauge();
	end
	if (type == "Static" or type == "static") then
		newWidget = CreateStaticText();
	end
	if (type == "StaticImage" or type == "image") then
		newWidget = CreateStaticImage();
	end
	if (type == "ListBox" or type == "listbox") then
		newWidget = CreateListBox();
	end
	if (type == "ListViewEx" or type == "listviewex") then
		newWidget = CreateListViewEx();
	end
	if (type == "Scrollable" or type == "scrollable") then
		newWidget = CreateScrollable();
	end
	if (type == "ColorSelect" or type == "colorselect") then
		newWidget = CreateColorSelect();
	end
	if (type == "Spin" or type == "spin") then
		newWidget = CreateSpin();
	end
	if (type == "Browser" or type == "browser") then
		newWidget = CreateBrowser();
	end
	if (newWidget ~= nil) then
		newWidget.name = name;
		_G[name] = newWidget;
		return newWidget;
	else
		return nil;
	end
end

------------------------------------------------------
-- Event Handler
------------------------------------------------------
function Widget:RegisterEvent(event)
	RegisterEvent(g_AddonName, event);
end

function Widget:UnregisterEvent(event)
	UnregisterEvent(g_AddonName, event);
end

function Widget:SetScript(event, script)
	self.triggerScript[event] = script;
end

function Widget:TriggerScript(triggerType, eventType, ...)
	-- Script function arg convention 'this' means current Widget, and 'event' means current event
	this	= self;
	event	= eventType;
	local scr = self.triggerScript[triggerType]
	if (scr == nil) then
		return;
	end
	if (type(scr) == "string") then
		assert(loadstring(scr))(...);
	elseif (type(scr) == "function") then
		scr(...);
	end
	this	= nil;
	event	= nil;
end

------------------------------------------------------
-- XML parser callback
------------------------------------------------------
function Widget:OnStartElement(name, attr)
	if (name == "OnLoad" or  name == "OnEvent") then
		CurrWalker.curTrigger = name;
	else
		CurrWalker.curTrigger = nil;
	end

	if (name == "Size") then
		CurrWalker.loadSize = true;
	end
	if (CurrWalker.loadSize) then
		if (name == "AbsDimension") then
			SetWidgetSize(g_AddonName, CurrWalker.name, attr.x, attr.y)
		end
	end
end

function Widget:OnEndElement(name)
	if (name == "Widget") then
		PopWalker();
	end
end

function Widget:OnCData(cdata)
	if (CurrWalker.curTrigger ~= nil) then
		CurrWalker.SetScript(CurrWalker, CurrWalker.curTrigger, cdata);
		CurrWalker.curTrigger = nil;
	end
end

------------------------------------------------------
-- lom parser functions
------------------------------------------------------
function Widget:LoadXML(xmlDoc)
	if (xmlDoc.attr ~= nil) then
		if (xmlDoc.attr.frame ~= nil) then
			local x, y, w, h = string.match(xmlDoc.attr.frame, "^(%d+)%s*,%s*(%d+)%s*,%s*(%d+)%s*,%s*(%d+)");
			if (x ~= nil and y ~= nil and w ~= nil and h ~= nil) then
				SetWidgetRect(g_AddonName, self.name, x, y, w, h);
			end
		end
		if (xmlDoc.attr.preset ~= nil) then
			self:SetPreset(xmlDoc.attr.preset);
		end
		if (xmlDoc.attr.text ~= nil) then
			self:SetText(xmlDoc.attr.text);
		end
		if (xmlDoc.attr.textId ~= nil) then
			self:SetText(GetAionStr(xmlDoc.attr.textId));
		end
		if (xmlDoc.attr.flag ~= nil) then
			self:SetFlag(xmlDoc.attr.flag);
		end
		if (xmlDoc.attr.HAlign ~= nil) then
			self:SetHAlign(xmlDoc.attr.HAlign);
		end
		if (xmlDoc.attr.VAlign ~= nil) then
			self:SetVAlign(xmlDoc.attr.VAlign);
		end
		if (xmlDoc.attr.padding ~= nil) then
			self:SetPadding(xmlDoc.attr.padding);
		end
		if (xmlDoc.attr.space ~= nil) then
			self:SetSpace(xmlDoc.attr.space);
		end
		if (xmlDoc.attr.numeric ~= nil) then
			self:SetNumeric(xmlDoc.attr.numeric);
		end
		if (xmlDoc.attr.maxLength ~= nil) then
			self:SetMaxLength(xmlDoc.attr.maxLength);
		end
		if (xmlDoc.attr.scroll_type ~= nil) then
			self:SetScrollType(xmlDoc.attr.scroll_type);
		end
		if (xmlDoc.attr.close_offset ~= nil) then
			local x, y = string.match(xmlDoc.attr.close_offset, "^(%d+)%s*,%s*(%d+)");
			self:SetCloseOffset(x, y);
		end
		if (xmlDoc.attr.tab_offset ~= nil) then
			self:SetAttr("tabOffset", xmlDoc.attr.tab_offset);
		end
		if (xmlDoc.attr.align_type ~= nil) then
			self:SetAlignType(xmlDoc.attr.align_type);
		end
		if (xmlDoc.attr.title_offset ~= nil) then
			local x, y = string.match(xmlDoc.attr.title_offset, "^(%d+)%s*,%s*(%d+)");
			self:SetTitleOffset(x, y);
		end
		if (xmlDoc.attr.title_height ~= nil) then
			self:SetTitleHeight(xmlDoc.attr.title_height);
		end
		if (xmlDoc.attr.font ~= nil) then
			self:SetFont(xmlDoc.attr.font);
		end
	end
	for i,node in ipairs(xmlDoc) do
		if (node.tag) then
			if (node.tag == "Scripts") then
				self:LoadXMLScripts(node);
			end
			if (node.tag == "Size") then
				self:LoadXMLSize(node);
			end
			if (node.tag == "Anchors") then
				self:LoadXMLAnchors(node);
			end
			if (node.tag == "Widgets") then
				self:LoadChildWidgets(node);
			end
			-- Resource Editor ȣȯ  ڵ.
			if (node.tag == "Widget") then
				self:LoadChildWidget(node);
			end
		end
	end
end

function Widget:LoadXMLScripts(xmlDoc)
	for i,node in ipairs(xmlDoc) do
		if (node ~= nil and node.tag ~= nil) then
			self:SetScript(node.tag, node[1]);
		end
	end
end

function GetAbsDimension(xmlDoc)
	for i, node in ipairs(xmlDoc) do
		if (node.tag ~= nil and node.tag == "AbsDimension") then
			return node.attr.x, node.attr.y;
		end
	end
end

function Widget:LoadXMLSize(xmlDoc)
	x, y = GetAbsDimension(xmlDoc);
	if (x ~= nil and y~= nil) then
		SetWidgetSize(g_AddonName, self.name, x, y);
	end
end

function GetAnchorOffset(xmlDoc)
	for i, node in ipairs(xmlDoc) do
		if (node.tag ~= nil and node.tag == "Offset") then
			return node.attr.x, node.attr.y;
		end
	end
end

function Widget:LoadXMLAnchors(xmlDoc)
	for i, node in ipairs(xmlDoc) do
		if (node.tag ~= nil and node.tag == "Anchor") then
			x, y = GetAnchorOffset(node);
			if (x ~= nil and y ~= nil) then
				SetWidgetPos(g_AddonName, self.name, x, y);
			end
		end
	end
end

function Widget:LoadChildWidgets(xmlDoc)
	for i, node in ipairs(xmlDoc) do
		if (node.tag) then
			childWidget = CreateWidget(node.tag, node.attr.name);
			if (childWidget) then
				AddWidgetChildWidget(g_AddonName, self.name, node.attr.name);
				childWidget:LoadXML(node);
			end
		end
	end
end

function Widget:LoadChildWidget(node)
	if (node.tag) then
		childWidget = CreateWidget(node.attr.type, node.attr.name, node.attr.style);
		if (childWidget) then
			AddWidgetChildWidget(g_AddonName, self.name, node.attr.name);
			childWidget:LoadXML(node);
		end
	end
end

------------------------------------------------------
-- C API
------------------------------------------------------
function Widget_TriggerScript(WidgetName, triggerType, eventType, ...)
	targetWidget = _G[WidgetName];
	if (targetWidget ~= nil and targetWidget.TriggerScript ~= nil) then
		targetWidget:TriggerScript(triggerType, eventType, unpack(arg));
	end
end

function Widget:AskEditBox(paramName, title, msg)
	AskEditBox(g_AddonName, self.name, paramName, title, msg);
end

function Widget:Show()
	SetWidgetVisible(g_AddonName, self.name, 1);
end

function Widget:Hide()
	SetWidgetVisible(g_AddonName, self.name, 0);
end

function Widget:IsVisible()
	return GetWidgetVisible(g_AddonName, self.name);
end

function Widget:SetText(str)
	SetWidgetText(g_AddonName, self.name, str);
end

function Widget:GetText(text)
	return GetWidgetText(g_AddonName, self.name);
end

function Widget:GetTextWidth(text)
	return GetWidgetTextWidth(g_AddonName, self.name, text);
end

function Widget:SetSize(x, y)
	SetWidgetSize(g_AddonName, self.name, x, y);
end

function Widget:SetPos(x, y)
	SetWidgetPos(g_AddonName, self.name, x, y);
end

function Widget:SetPosForced(x, y)
	SetWidgetPosForced(g_AddonName, self.name, x, y);
end

function Widget:SetRect(x, y, width, height)
	SetWidgetRect(g_AddonName, self.name, x, y, width, height);
end

function Widget:GetRect()
	return GetWidgetRect(g_AddonName, self.name);
end

function Widget:SetPreset(preset)
	SetWidgetPreset(g_AddonName, self.name, preset);
end

function Widget:SetFlag(flag)
	SetWidgetFlag(g_AddonName, self.name, flag);
end

function Widget:GetFlag(flag)
	return GetWidgetFlag(g_AddonName, self.name, flag);
end


function Widget:SetAttr(attr, value)
	SetWidgetAttr(g_AddonName, self.name, attr, value);
end

function Widget:GetAttr(attr)
	return GetWidgetAttr(g_AddonName, self.name, attr);
end

function Widget:Enable(bEnabled)
	if (bEnabled) then
		SetWidgetEnable(g_AddonName, self.name, 1);
	else
		SetWidgetEnable(g_AddonName, self.name, 0);
	end
end


function Widget:SetTooltip(tooltip)
	SetWidgetTooltip(g_AddonName, self.name, tooltip);
end


function Widget:SetFocus()
	SetWidgetFocus(g_AddonName, self.name);
end


function Widget:SetValue(v)
	SetWidgetValue(g_AddonName, self.name, v);
end

function Widget:GetValue()
	return GetWidgetValue(g_AddonName, self.name);
end

function Widget:SetMaxValue(value)
	SetWidgetMaxValue(g_AddonName, self.name, value);
end


function Widget:SetPadding(x, y)
	SetWidgetPadding(g_AddonName, self.name, x, y);
end

function Widget:SetHAlign(align)
	SetWidgetHAlign(g_AddonName, self.name, align);
end

function Widget:SetVAlign(align)
	SetWidgetVAlign(g_AddonName, self.name, align);
end

function Widget:SetSelectionRect(left, top, width, height)
	SetWidgetSelectionRect(g_AddonName, self.name, left, top, width, height);
end


function Widget:SetSpace(space)
	SetWidgetSpace(g_AddonName, self.name, space);
end

function Widget:SetNumeric(numeric)
	SetWidgetNumeric(g_AddonName, self.name, numeric);
end

function Widget:SetMaxLength(length)
	SetWidgetMaxLength(g_AddonName, self.name, length);
end

function Widget:SetScrollType(scrollType)
	SetWidgetScrollType(g_AddonName, self.name, scrollType);
end

function Widget:SetCloseOffset(x, y)
	SetWidgetCloseOffset(g_AddonName, self.name, x, y);
end

function Widget:SetTitleOffset(x, y)
	SetWidgetTitleOffset(g_AddonName, self.name, x, y);
end

function Widget:SetTitleHeight(height)
	SetWidgetTitleHeight(g_AddonName, self.name, height);
end

function Widget:SetFont(font)
	SetWidgetFont(g_AddonName, self.name, font);
end

function Widget:SetFrameSelectionRect(rect)
	SetWidgetSelectionRect(g_AddonName, self.name, rect);
end


function Widget:AddItem(str)
	AddWidgetItem(g_AddonName, self.name, str);
end

function Widget:AddItemWithTooltip(str, tooltip)
	AddWidgetItemWithTooltip(g_AddonName, self.name, str, tooltip);
end

function Widget:SetCell(itemid, colid, str, tooltip)
	SetWidgetCell(g_AddonName, self.name, itemid, colid, str, tooltip);
end

function Widget:DeleteItem(index)
	DeleteWidgetItem(g_AddonName, self.name, index);
end

function Widget:ClearItem()
	ClearWidgetItem(g_AddonName, self.name);
end


function Widget:SetCaptureMouse()
	SetWidgetCaptureMouse(g_AddonName, self.name);
end

function Widget:SetReleaseMouse()
	SetWidgetReleaseMouse(g_AddonName, self.name);
end


function Widget:SetTimer(remainTime)
	SetTimer(g_AddonName, remainTime);
end

function Widget:ResetTimer()
	ResetTimer(g_AddonName);
end

function Widget:SetVScrollValue(v)
	SetWidgetVScrollValue(g_AddonName, self.name, v);
end

function Widget:GetVScrollValue()
	return GetWidgetVScrollValue(g_AddonName, self.name);
end

function Widget:SetListViewParams(cellPadding, listPadding, itemHeight, headerHeight, overColor, selectionColor, columnHeading, headerFont, useCheck, customizing)
	return SetWidgetListViewParams(g_AddonName, self.name, cellPadding, listPadding, itemHeight, headerHeight, overColor, selectionColor, columnHeading, headerFont, useCheck, customizing);
end

function Widget:AddColumn(name, alignType, sortType, width)
	return AddWidgetColumn(g_AddonName, self.name, name, alignType, sortType, width);
end

function Widget:AddOptionList(variableName)
	AddOptionList(g_AddonName, variableName);
end

function Widget:SetAlignType(alignType)
	SetWidgetAlignType(g_AddonName, self.name, alignType);
end

function Widget:AddFilter(filter)
	AddWidgetFilter(g_AddonName, self.name, filter);
end

function Widget:CreateWebView()
	CreateWebView(g_AddonName, self.name);
end

function Widget:DestroyWebView()
	DestroyWebView(g_AddonName, self.name);
end

function Widget:GetWebViewIndex()
	return GetWebViewIndex(g_AddonName, self.name);
end

function Widget:GetTextureId()
	return GetTextureId(g_AddonName, self.name);
end

function Widget:SetCookie(url, cookieStr)
	SetCookie(g_AddonName, self.name, url, cookieStr);
end

function Widget:ClearCookies()
	ClearCookies(g_AddonName, self.name);
end

function Widget:LoadUrlWithWebAuth(url)
	LoadUrlWithWebAuth(g_AddonName, self.name, url);
end

function Widget:UseOpenCloseFx(bUse)
	if (bUse) then
		SetWidgetOpenCloseFx(g_AddonName, self.name, 1);
	else
		SetWidgetOpenCloseFx(g_AddonName, self.name, 0);
	end
end

function Widget:SetOpenCloseFxRect(x, y, width, height)
	SetWidgetOpenCloseFxRect(g_AddonName, self.name, x, y, width, height);
end

function Widget:GetTextureName()
	return GetWidgetTextureName(g_AddonName, self.name);
end

function Widget:SaveTexture(path)
	return SaveWidgetTexture(g_AddonName, self.name, path);
end

function Widget:SetUIImage(state, imagePath)
	SetWidgetUIImage(g_AddonName, self.name, state, imagePath);
end

function Widget:SetUIImageById(state, id)
	SetWidgetUIImageById(g_AddonName, self.name, state, id);
end

function Widget:SetAlpha(alpha, changeChildWidget)
	local ccw = 0;
	if (changeChildWidget) then
		ccw = 1;
	end
	SetWidgetAlpha(g_AddonName, self.name, alpha, ccw);
end

function Widget:ScaleToDefault()
	ScaleWidgetToDefault(g_AddonName, self.name);
end

function Widget:RegisterHandleCancelKey()
	RegisterHandleCancelKeyFrame(g_AddonName, self.name);
end
