------------------------------------------------------
-- SAX XML parser callback
------------------------------------------------------
XMLHandler = { StartElement=nil, EndElement=false, CharacterData=false };
function SetCurrentWalker(walker)
	XMLHandler.StartElement		= walker.OnStartElement;
	XMLHandler.EndElement		= walker.OnEndElement;
	XMLHandler.CharacterData	= walker.OnCData;
	CurrWalker = walker;
end

------------------------------------------------------
-- XML Walker stack
------------------------------------------------------
XMLWalkers = {};
CurrWalker = nil;
for i=1, 100 do
	XMLWalkers[i] = {};
end

function PushWalker(oldWalker, newWalker)
	XMLWalkers[#XMLWalkers+1] = oldWalker;
	SetCurrentWalker(newWalker)
end

function PopWalker()
	size = #XMLWalkers;
	if (size > 1) then
		SetCurrentWalker(XMLWalkers[size-1]);
	end
	XMLWalkers[size] = nil;
end

------------------------------------------------------
-- class XMLDocument
------------------------------------------------------
XMLDocument = {};
function XMLDocument:new(o)
	o = o or {};
	setmetatable(o, self);
	self.__index = self;
	return o;
end
g_XMLDir = "";	-- Directory of XML document

------------------------------------------------------
-- SAX XML parser callback for XMLDocument
------------------------------------------------------
function XMLDocument:OnStartElement(name, attr)
	if (name == "Script") then
		dofile(g_XMLDir..attr.file)
	end
	if (name == "Widget") then
		newWidget = CreateWidget("Widget", attr.name);	-- _G[attr.name] = newWidget;
		PushWalker(CurrWalker, newWidget);
	end
end

function XMLDocument:OnCData(cdata)
	--placeholder
	--print ("XMLDocument:OnCData")
end

function XMLDocument:OnEndElement(name)
	--placeholder
	--print ("XMLDocument:OnEndElement")
end

------------------------------------------------------
-- Read XML file and parse it
-- return: err, line, col, pos, msg;
------------------------------------------------------
function LoadXML_(addonName, fileName)
	g_AddonName = addonName;
	local dir = "Plugin/" .. addonName .. "/"
	g_XMLDir = dir;
	local path = dir .. fileName;

	local f = assert(io.open(path, "r"));
	if (f == nil) then
		return -1, 0, 0, 0, "LoadXML() : cannot open file "..path;
	end
	local t = f:read("*all");
	f:close();

	newDoc = XMLDocument:new();
	SetCurrentWalker(newDoc);

	local p = lxp.new(XMLHandler);
	err, msg, line, col, pos = p:parse(t);
	if (err ~= nil) then
		-- todo: error
		return err, line, col, pos, msg;
	end
	p:close();
	return 0, 0, 0, 0, "";
end

------------------------------------------------------
-- lom parser functions
------------------------------------------------------
function LoadXML(addonName, fileName)
	g_AddonName = addonName;
	local dir = "Plugin/" .. addonName .. "/"
	g_XMLDir = dir;
	local path = dir .. fileName;

	--local f = assert(io.open(path, "r"));
	local f = io.open(path, "r");
	if (f == nil) then
		dir = "PluginCache/" .. addonName .. "/"
		g_XMLDir = dir;
		path = dir .. fileName;
		f = io.open(path, "r");
		if (f == nil) then
			return -1, 0, 0, 0, "LoadXML() : cannot open file "..path;
		end
	end
	local t = f:read("*all");
	f:close();

	newDoc = lxp.lom.parse(t);
	if (newDoc == nil) then
		return -1, 0, 0, 0, "LoadXML() : cannot parse file "..path;
	end
	for i,node in ipairs(newDoc) do
		-- if (node.tag == "Script") then
		--	dofile(dir..node.attr.file);
		-- end
		if (node.tag == "Dialog") then
			LoadXMLWidget(node)
		end
	end
end

function LoadXMLWidget(xmlDoc)
	local WidgetName = xmlDoc.attr.name;
	newWidget = CreateWidget("Dialog", WidgetName);
	newWidget:LoadXML(xmlDoc);
end
